@@ -1,12 +1,19 @@
 import os
 from typing import List
-
 import matplotlib
 import matplotlib.pyplot as plt
+from mpl_toolkits.axes_grid1 import make_axes_locatable
 import numpy as np
 
 from py2lispIDyOM.extract import MelodyInfo, ExperimentInfo
 
+# style customization:
+
+matplotlib.rcParams['xtick.labelsize'] = 16
+matplotlib.rcParams['ytick.labelsize'] = 16
+matplotlib.rcParams['axes.labelsize'] = 16
+matplotlib.rcParams['figure.titlesize'] = 20
+
 
 class BasicAxsGeneration:
 
@@ -141,7 +148,6 @@ class BasicAxsGeneration:
         yticks = np.arange(0, max(max_ic, max_entropy) + 1)
 
         ax.set_xticks(ticks=xticks, minor=True)
-        ax.set_xlabel('Time in quarter note')
         ax.set_yticks(ticks=yticks, minor=True)
         ax.legend()
         if grid is True:
@@ -176,12 +182,11 @@ class BasicAxsGeneration:
         colored_image[background_mask] = np.array([68, 1, 84])
         colored_image = colored_image.astype(int)
 
-        ax.imshow(colored_image, origin='lower', aspect='auto',
-                  extent=[0, duration_in_ticks / 24, pitch_min, pitch_max])
         # ax.axis('image')
-        # ax.set_xlabel("Time in quarter note")
         # ax.xaxis.set_ticklabels([])  # hide xtick labels
-        ax.set_ylabel('Pitch (MIDI number)', fontsize=10)
+        ax.set_ylabel('Pitch (MIDI number)')
+        ax = ax.imshow(colored_image, origin='lower', aspect='auto',
+                       extent=[0, duration_in_ticks / 24, pitch_min, pitch_max])
         return ax
 
     @staticmethod
@@ -192,12 +197,32 @@ class BasicAxsGeneration:
 
         duration_in_ticks = melody_info._get_pianoroll_original().shape[1]
         pianoroll_distribution_array = melody_info._get_pianoroll_pitch_distribution()
-        ax.imshow(pianoroll_distribution_array, origin='lower', aspect='auto',
-                  extent=[0, duration_in_ticks / 24, pitch_min, pitch_max])
+        ax.set_ylabel('Pitch (MIDI number)')
+        ax = ax.imshow(pianoroll_distribution_array, origin='lower', aspect='auto',
+                       extent=[0, duration_in_ticks / 24, pitch_min, pitch_max])
 
-        # ax.title.set_text('Pitch Prediction')
-        # ax.set_xlabel("Time in quarter note")
-        ax.set_ylabel('Pitch (MIDI number)', fontsize=10)
+        return ax
+
+    @staticmethod
+    def ax_suprisal_colorbar(ax: matplotlib.axes.Axes,
+                             melody_info: MelodyInfo,
+                             color='viridis'):
+        surprisal_values = melody_info.get_idyom_output_nparray('information.content')
+        # print(surprisal_values)
+        # extent = [min(surprisal_values), max(surprisal_values)]
+        # ax.set_xticks(np.arange(*(extent + [1])), minor=True)
+        ax.set_xlabel('Information content (surprisal) values')
+        ax = ax.imshow(X=surprisal_values.reshape(1, -1), cmap=color, origin='lower')
+        return ax
+
+    @staticmethod
+    def ax_blank_colorbar(ax: matplotlib.axes.Axes,
+                          melody_info: MelodyInfo,
+                          color='gray'):
+        surprisal_values = melody_info.get_idyom_output_nparray('information.content')
+        seq_len = len(surprisal_values)
+        blank_seq = np.zeros(seq_len)
+        ax = ax.imshow(X=blank_seq.reshape(1, -1), cmap=color, origin='lower')
         return ax
 
     @staticmethod
@@ -205,8 +230,8 @@ class BasicAxsGeneration:
                 melody_info: MelodyInfo,
                 color=None):
 
-        entropy_values = np.int_(melody_info.access_properties(['entropy']))
-        onset_values = np.int_(melody_info.access_properties(['onset']))
+        entropy_values = melody_info.get_idyom_output_nparray('entropy')
+        onset_values = melody_info.get_idyom_output_nparray('onset')
         ax.plot(onset_values, entropy_values, color=color)
         ax.margins(x=0.01)
         ax.margins(y=0)
@@ -214,7 +239,7 @@ class BasicAxsGeneration:
         ax.spines['right'].set_visible(False)
         ax.set_xlabel("Time in quarter note")
         #    ax.xaxis.set_ticklabels([])  # hide xtick labels
-        ax.set_ylabel('Entropy', fontsize=10)
+        ax.set_ylabel('Entropy')
 
         return ax
 
@@ -223,14 +248,14 @@ class BasicAxsGeneration:
                   melody_info: MelodyInfo,
                   color=None):
 
-        surprisal_values = np.int_(melody_info.access_properties(['information.content']))
-        onset_values = np.int_(melody_info.access_properties(['onset']))
+        surprisal_values = melody_info.get_idyom_output_nparray('information.content')
+        onset_values = melody_info.get_idyom_output_nparray('onset')
         ax.plot(onset_values, surprisal_values, color=color)
         ax.margins(x=0.01)
         ax.margins(y=0)
         ax.spines['top'].set_visible(False)
         ax.spines['right'].set_visible(False)
-        ax.set_ylabel('Information Content (Surprisal) \n -log(P)', fontsize=10)
+        ax.set_ylabel('Information Content (Surprisal) \n -log(P)')
         return ax
 
     @staticmethod
@@ -248,7 +273,7 @@ class BasicAxsGeneration:
         ax.spines['right'].set_visible(False)
         # ax.set_xlabel('Time in quarter note')
         #    ax.xaxis.set_ticklabels([])  # hide xtick labels
-        ax.set_ylabel('Information Content (Surprisal) \n -log(P)', fontsize=10)
+        ax.set_ylabel('Information Content (Surprisal) \n -log(P)')
         return ax
 
 
@@ -323,7 +348,8 @@ class BasicPlot:
                                                dpi: float = 400,
                                                figsize: tuple = (10, 10),
                                                nrows: int = 2,
-                                               ncols: int = 1, ):
+                                               ncols: int = 1,
+                                               probability_colorbar: bool = False):
 
         """
         Generate a pair of figures (the predicted pitch distribution and the ground truth) side by side.
@@ -341,31 +367,44 @@ class BasicPlot:
         :param figsize: (optional), default is (10,5)
         :param ncols: (optional), the number of columns of the figure. By default, ncols = 2, nrows = 1, figures are shown side-by-side.
         :param nrows: (optional), the number of columns of the figure. By default, ncols = 2, nrows = 1, figures are shown side-by-side.
-
+        :param probability_colorbar: (optional) whether to show the color bar for the probabilities of the predict pitch or not.
         """
 
         plot_type_folder_name = 'pianoroll_pitch_prediction_groundtruth'
 
         def _pianoroll_pitch_prediction_groundtruth(melody_info: MelodyInfo,
-                                                    show_single_fig: bool = showfig) -> plt.Figure:
+                                                    show_single_fig: bool = showfig,
+                                                    probability_colorbar: bool = probability_colorbar) -> plt.Figure:
 
             melody_name_pprint = melody_info._get_melody_name_pprint()
 
             fig, (ax_distribution, ax_groundtruth) = plt.subplots(nrows=nrows, ncols=ncols, figsize=figsize, dpi=dpi)
+            fig.subplots_adjust(wspace=0.03)
             fig.suptitle('IDyOM Pitch prediction vs Ground truth \n\n Melody name: ' + melody_name_pprint)
 
-            BasicAxsGeneration.pianoroll_pitch_distribution(ax_distribution, melody_info=melody_info)
-            BasicAxsGeneration.pianoroll(ax_groundtruth, melody_info=melody_info)
-            # ax_groundtruth.title.set_text('Ground Truth')
+            distribution = BasicAxsGeneration.pianoroll_pitch_distribution(ax_distribution, melody_info=melody_info)
+            groundtruth = BasicAxsGeneration.pianoroll(ax_groundtruth, melody_info=melody_info)
 
-            plt.xlabel("Time in quarter note")
+            if probability_colorbar is True:
+                distribution_divider = make_axes_locatable(ax_distribution)
+                ax_surprise = distribution_divider.append_axes("right", size="3%", pad=0.1)
+                cb_surprisal = fig.colorbar(distribution, cax=ax_surprise)
+                cb_surprisal.set_label('Probabilities of predicted pitches')
+
+                truth_divider = make_axes_locatable(ax_groundtruth)
+                ax_blank = truth_divider.append_axes("right", size="3%", pad=0.1)
+                # cb_blank = fig.colorbar(groundtruth, cax=ax_blank)
+                ax_blank.axis('off')
+            else:
+                pass
+
+            fig.supxlabel("Time in quarter note")
             plt.tight_layout()
 
             if show_single_fig is True:
                 plt.show()
             else:
                 pass
-
             return fig
 
         Auxiliary.batch_melodies_plots(plot_method_func=_pianoroll_pitch_prediction_groundtruth,
@@ -412,12 +451,13 @@ class BasicPlot:
             melody_name_pprint = melody_info._get_melody_name_pprint()
 
             fig, (ax_pianoroll, ax_surprisal) = plt.subplots(2, 1, figsize=figsize, dpi=dpi, sharex='col')
+            fig.subplots_adjust(wspace=0.03)
             fig.suptitle('IDyOM Information Content (Surprisal) \n\n Melody: ' + melody_name_pprint)
 
             BasicAxsGeneration.pianoroll(ax_pianoroll, melody_info=melody_info)
             BasicAxsGeneration.surprisal_continuous(ax_surprisal, melody_info=melody_info)
 
-            plt.xlabel("Time in quarter note")
+            fig.supxlabel("Time in quarter note")
             plt.tight_layout()
 
             if show_single_fig is True:
@@ -481,8 +521,7 @@ class BasicPlot:
             melody_name_pprint = melody_info._get_melody_name_pprint()
             fig, (ax_generic_property) = plt.subplots(nrows=1, ncols=1, figsize=figsize, dpi=dpi, sharex='col')
 
-            fig.suptitle('Selected IDyOM outputs: ' + selected_idyom_output + '\n\n Melody: ' + melody_name_pprint,
-                         fontsize=15)
+            fig.suptitle('Selected IDyOM outputs: ' + selected_idyom_output + '\n\n Melody: ' + melody_name_pprint)
             BasicAxsGeneration.generic_idyom_output_along_time(ax_generic_property,
                                                                melody_info=melody_info,
                                                                selected_idyom_output=selected_idyom_output,
@@ -562,7 +601,8 @@ class BasicPlot:
                                                                 ic_color='C0',
                                                                 entropy_color='C1',
                                                                 grid=grid)
-
+            fig.supxlabel('Time in quarter note')
+            fig.supylabel('Surprisal or entropy values')
             plt.tight_layout()
 
             if show_single_fig is True:
@@ -640,6 +680,7 @@ class BasicPlot:
             # plot a number of surprisal plots stacked one on top of each other, shared x, y
             # add every single subplot to the figure with a for loop
             fig, axs = plt.subplots(num_of_surprisal_subplots, figsize=figsize, dpi=dpi, sharex='col', sharey='row')
+            fig.subplots_adjust(wspace=0.03)
             fig.suptitle('IDyOM Information Content (Surprisal) \n\n Melody: ' + melody_name_pprint)
 
             for idx, surprisal_type in enumerate(all_surprisal_sources):
@@ -650,7 +691,9 @@ class BasicPlot:
                                                        color=str('C' + str(idx)),
                                                        yticks=yticks)
 
-            plt.xlabel("Time in quarter note")
+            fig.supxlabel("Time in quarter note")
+            fig.supylabel(t='Information Content (Surprisal)\n-log(P)', x=0.04, ha='center')
+
             plt.tight_layout()
             plt.style.use('ggplot')
 
